@@ -21,6 +21,7 @@ from random import random
 
 import os
 import six
+from six.moves.urllib.parse import quote
 from eventlet import Timeout
 
 from swift.common import internal_client
@@ -40,6 +41,11 @@ from swift.container.backend import ContainerBroker, \
 from swift.container.replicator import ContainerReplicator
 
 
+CLEAVE_SUCCESS = 0
+CLEAVE_FAILED = 1
+CLEAVE_EMPTY = 2
+
+
 def sharding_enabled(broker):
     # NB all shards will by default have been created with
     # X-Container-Sysmeta-Sharding set and will therefore be candidates for
@@ -307,6 +313,12 @@ class CleavingContext(object):
         self.cleaving_done = False
         self.cleave_to_row = self.max_row
 
+    def range_done(self, new_cursor):
+        self.ranges_done += 1
+        self.ranges_todo -= 1
+        if new_cursor is not None:
+            self.cursor = new_cursor
+
     def done(self):
         return all((self.misplaced_done, self.cleaving_done,
                     self.max_row == self.cleave_to_row))
@@ -394,8 +406,9 @@ class ContainerSharder(ContainerReplicator):
                 'Swift Container Sharder',
                 request_tries,
                 allow_modify_pipeline=False)
-        except IOError as err:
-            if err.errno != errno.ENOENT:
+        except (OSError, IOError) as err:
+            if err.errno != errno.ENOENT and \
+                    not str(err).endswith(' not found'):
                 raise
             raise SystemExit(
                 'Unable to load internal client from config: %r (%s)' %
@@ -558,13 +571,13 @@ class ContainerSharder(ContainerReplicator):
                     params=params)
             except internal_client.UnexpectedResponse as err:
                 self.logger.warning("Failed to get shard ranges from %s: %s",
-                                    broker.root_path, err)
+                                    quote(broker.root_path), err)
                 return None
             record_type = resp.headers.get('x-backend-record-type')
             if record_type != 'shard':
                 err = 'unexpected record type %r' % record_type
                 self.logger.error("Failed to get shard ranges from %s: %s",
-                                  broker.root_path, err)
+                                  quote(broker.root_path), err)
                 return None
 
             try:
@@ -576,7 +589,7 @@ class ContainerSharder(ContainerReplicator):
             except (ValueError, TypeError, KeyError) as err:
                 self.logger.error(
                     "Failed to get shard ranges from %s: invalid data: %r",
-                    broker.root_path, err)
+                    quote(broker.root_path), err)
             return None
         finally:
             self.logger.txn_id = None
@@ -601,7 +614,8 @@ class ContainerSharder(ContainerReplicator):
 
     def _send_shard_ranges(self, account, container, shard_ranges,
                            headers=None):
-        body = json.dumps([dict(sr) for sr in shard_ranges]).encode('ascii')
+        body = json.dumps([dict(sr, reported=0)
+                           for sr in shard_ranges]).encode('ascii')
         part, nodes = self.ring.get_nodes(account, container)
         headers = headers or {}
         headers.update({'X-Backend-Record-Type': RECORD_TYPE_SHARD,
@@ -635,25 +649,33 @@ class ContainerSharder(ContainerReplicator):
         """
         part = self.ring.get_part(shard_range.account, shard_range.container)
         node = self.find_local_handoff_for_part(part)
+        put_timestamp = Timestamp.now().internal
         if not node:
             raise DeviceUnavailable(
                 'No mounted devices found suitable for creating shard broker '
-                'for %s in partition %s' % (shard_range.name, part))
+                'for %s in partition %s' % (quote(shard_range.name), part))
 
         shard_broker = ContainerBroker.create_broker(
             os.path.join(self.root, node['device']), part, shard_range.account,
             shard_range.container, epoch=shard_range.epoch,
-            storage_policy_index=policy_index)
+            storage_policy_index=policy_index, put_timestamp=put_timestamp)
 
         # Get the valid info into the broker.container, etc
         shard_broker.get_info()
         shard_broker.merge_shard_ranges(shard_range)
-        shard_broker.set_sharding_sysmeta('Root', root_path)
+        shard_broker.set_sharding_sysmeta('Quoted-Root', quote(root_path))
+        # NB: we *used* to do
+        #    shard_broker.set_sharding_sysmeta('Root', root_path)
+        # but that isn't safe for container names with nulls or newlines (or
+        # possibly some other characters). We consciously *don't* make any
+        # attempt to set the old meta; during an upgrade, some shards may think
+        # they are in fact roots, but it cleans up well enough once everyone's
+        # upgraded.
         shard_broker.update_metadata({
             'X-Container-Sysmeta-Sharding':
                 ('True', Timestamp.now().internal)})
 
-        return part, shard_broker, node['id']
+        return part, shard_broker, node['id'], put_timestamp
 
     def _audit_root_container(self, broker):
         # This is the root container, and therefore the tome of knowledge,
@@ -663,7 +685,8 @@ class ContainerSharder(ContainerReplicator):
         own_shard_range = broker.get_own_shard_range()
 
         if own_shard_range.state in (ShardRange.SHARDING, ShardRange.SHARDED):
-            shard_ranges = broker.get_shard_ranges()
+            shard_ranges = [sr for sr in broker.get_shard_ranges()
+                            if sr.state != ShardRange.SHRINKING]
             missing_ranges = find_missing_ranges(shard_ranges)
             if missing_ranges:
                 warnings.append(
@@ -672,6 +695,10 @@ class ContainerSharder(ContainerReplicator):
                               for lower, upper in missing_ranges]))
 
         for state in ShardRange.STATES:
+            if state == ShardRange.SHRINKING:
+                # Shrinking is how we resolve overlaps; we've got to
+                # allow multiple shards in that state
+                continue
             shard_ranges = broker.get_shard_ranges(states=state)
             overlaps = find_overlapping_ranges(shard_ranges)
             for overlapping_ranges in overlaps:
@@ -683,8 +710,8 @@ class ContainerSharder(ContainerReplicator):
 
         if warnings:
             self.logger.warning(
-                'Audit failed for root %s (%s): %s' %
-                (broker.db_file, broker.path, ', '.join(warnings)))
+                'Audit failed for root %s (%s): %s',
+                broker.db_file, quote(broker.path), ', '.join(warnings))
             self._increment_stat('audit_root', 'failure', statsd=True)
             return False
 
@@ -692,7 +719,6 @@ class ContainerSharder(ContainerReplicator):
         return True
 
     def _audit_shard_container(self, broker):
-        # Get the root view of the world.
         self._increment_stat('audit_shard', 'attempted')
         warnings = []
         errors = []
@@ -702,8 +728,10 @@ class ContainerSharder(ContainerReplicator):
 
         own_shard_range = broker.get_own_shard_range(no_default=True)
 
-        shard_range = None
+        shard_ranges = own_shard_range_from_root = None
         if own_shard_range:
+            # Get the root view of the world, at least that part of the world
+            # that overlaps with this shard's namespace
             shard_ranges = self._fetch_shard_ranges(
                 broker, newest=True,
                 params={'marker': str_to_wsgi(own_shard_range.lower_str),
@@ -711,44 +739,76 @@ class ContainerSharder(ContainerReplicator):
                 include_deleted=True)
             if shard_ranges:
                 for shard_range in shard_ranges:
-                    if (shard_range.lower == own_shard_range.lower and
-                            shard_range.upper == own_shard_range.upper and
-                            shard_range.name == own_shard_range.name):
+                    # look for this shard range in the list of shard ranges
+                    # received from root; the root may have different lower and
+                    # upper bounds for this shard (e.g. if this shard has been
+                    # expanded in the root to accept a shrinking shard) so we
+                    # only match on name.
+                    if shard_range.name == own_shard_range.name:
+                        own_shard_range_from_root = shard_range
                         break
                 else:
                     # this is not necessarily an error - some replicas of the
                     # root may not yet know about this shard container
                     warnings.append('root has no matching shard range')
-                    shard_range = None
-            else:
+            elif not own_shard_range.deleted:
                 warnings.append('unable to get shard ranges from root')
+            # else, our shard range is deleted, so root may have reclaimed it
         else:
             errors.append('missing own shard range')
 
         if warnings:
             self.logger.warning(
-                'Audit warnings for shard %s (%s): %s' %
-                (broker.db_file, broker.path, ', '.join(warnings)))
+                'Audit warnings for shard %s (%s): %s',
+                broker.db_file, quote(broker.path), ', '.join(warnings))
 
         if errors:
             self.logger.warning(
-                'Audit failed for shard %s (%s) - skipping: %s' %
-                (broker.db_file, broker.path, ', '.join(errors)))
+                'Audit failed for shard %s (%s) - skipping: %s',
+                broker.db_file, quote(broker.path), ', '.join(errors))
             self._increment_stat('audit_shard', 'failure', statsd=True)
             return False
 
-        if shard_range:
-            self.logger.debug('Updating shard from root %s', dict(shard_range))
-            broker.merge_shard_ranges(shard_range)
+        if own_shard_range_from_root:
+            # iff we find our own shard range in the root response, merge it
+            # and reload own shard range (note: own_range_from_root may not
+            # necessarily be 'newer' than the own shard range we already have,
+            # but merging will get us to the 'newest' state)
+            self.logger.debug('Updating own shard range from root')
+            broker.merge_shard_ranges(own_shard_range_from_root)
+            orig_own_shard_range = own_shard_range
             own_shard_range = broker.get_own_shard_range()
-            delete_age = time.time() - self.reclaim_age
-            if (own_shard_range.state == ShardRange.SHARDED and
-                    own_shard_range.deleted and
-                    own_shard_range.timestamp < delete_age and
-                    broker.empty()):
-                broker.delete_db(Timestamp.now().internal)
-                self.logger.debug('Deleted shard container %s (%s)',
-                                  broker.db_file, broker.path)
+            if (orig_own_shard_range != own_shard_range or
+                    orig_own_shard_range.state != own_shard_range.state):
+                self.logger.debug(
+                    'Updated own shard range from %s to %s',
+                    orig_own_shard_range, own_shard_range)
+            if own_shard_range.state in (ShardRange.SHRINKING,
+                                         ShardRange.SHRUNK):
+                # If the up-to-date state is shrinking, save off *all* shards
+                # returned because these may contain shards into which this
+                # shard is to shrink itself; shrinking is the only case when we
+                # want to learn about *other* shard ranges from the root.
+                # We need to include shrunk state too, because one replica of a
+                # shard may already have moved the own_shard_range state to
+                # shrunk while another replica may still be in the process of
+                # shrinking.
+                other_shard_ranges = [sr for sr in shard_ranges
+                                      if sr is not own_shard_range_from_root]
+                self.logger.debug('Updating %s other shard range(s) from root',
+                                  len(other_shard_ranges))
+                broker.merge_shard_ranges(other_shard_ranges)
+
+        delete_age = time.time() - self.reclaim_age
+        deletable_states = (ShardRange.SHARDED, ShardRange.SHRUNK)
+        if (own_shard_range.state in deletable_states and
+                own_shard_range.deleted and
+                own_shard_range.timestamp < delete_age and
+                broker.empty()):
+            broker.delete_db(Timestamp.now().internal)
+            self.logger.debug('Deleted shard container %s (%s)',
+                              broker.db_file, quote(broker.path))
+
         self._increment_stat('audit_shard', 'success', statsd=True)
         return True
 
@@ -840,7 +900,7 @@ class ContainerSharder(ContainerReplicator):
             last_index = next_index = 0
             for obj in objs:
                 if dest_shard_range is None:
-                    # no more destinations: yield remainder of batch and return
+                    # no more destinations: yield remainder of batch and bail
                     # NB there may be more batches of objects but none of them
                     # will be placed so no point fetching them
                     yield objs[last_index:], None, info
@@ -883,7 +943,7 @@ class ContainerSharder(ContainerReplicator):
         if not success and responses.count(True) < quorum:
             self.logger.warning(
                 'Failed to sufficiently replicate misplaced objects: %s in %s '
-                '(not removing)', dest_shard_range, broker.path)
+                '(not removing)', dest_shard_range, quote(broker.path))
             return False
 
         if broker.get_info()['id'] != info['id']:
@@ -903,7 +963,7 @@ class ContainerSharder(ContainerReplicator):
         if not success:
             self.logger.warning(
                 'Refused to remove misplaced objects: %s in %s',
-                dest_shard_range, broker.path)
+                dest_shard_range, quote(broker.path))
         return success
 
     def _move_objects(self, src_broker, src_shard_range, policy_index,
@@ -929,7 +989,7 @@ class ContainerSharder(ContainerReplicator):
                 continue
 
             if dest_shard_range not in dest_brokers:
-                part, dest_broker, node_id = self._get_shard_broker(
+                part, dest_broker, node_id, _junk = self._get_shard_broker(
                     dest_shard_range, src_broker.root_path, policy_index)
                 # save the broker info that was sampled prior to the *first*
                 # yielded objects for this destination
@@ -946,7 +1006,7 @@ class ContainerSharder(ContainerReplicator):
         if unplaced:
             self.logger.warning(
                 'Failed to find destination for at least %s misplaced objects '
-                'in %s' % (unplaced, src_broker.path))
+                'in %s', unplaced, quote(src_broker.path))
 
         # TODO: consider executing the replication jobs concurrently
         for dest_shard_range, dest_args in dest_brokers.items():
@@ -1032,7 +1092,7 @@ class ContainerSharder(ContainerReplicator):
             their correct shard containers, False otherwise
         """
         self.logger.debug('Looking for misplaced objects in %s (%s)',
-                          broker.path, broker.db_file)
+                          quote(broker.path), broker.db_file)
         self._increment_stat('misplaced', 'attempted')
         src_broker = src_broker or broker
         if src_bounds is None:
@@ -1071,10 +1131,12 @@ class ContainerSharder(ContainerReplicator):
         own_shard_range = broker.get_own_shard_range()
         shard_ranges = broker.get_shard_ranges()
         if shard_ranges and shard_ranges[-1].upper >= own_shard_range.upper:
-            self.logger.debug('Scan already completed for %s', broker.path)
+            self.logger.debug('Scan for shard ranges already completed for %s',
+                              quote(broker.path))
             return 0
 
-        self.logger.info('Starting scan for shard ranges on %s', broker.path)
+        self.logger.info('Starting scan for shard ranges on %s',
+                         quote(broker.path))
         self._increment_stat('scanned', 'attempted')
 
         start = time.time()
@@ -1119,8 +1181,17 @@ class ContainerSharder(ContainerReplicator):
             shard_range.update_state(ShardRange.CREATED)
             headers = {
                 'X-Backend-Storage-Policy-Index': broker.storage_policy_index,
-                'X-Container-Sysmeta-Shard-Root': broker.root_path,
-                'X-Container-Sysmeta-Sharding': True}
+                'X-Container-Sysmeta-Shard-Quoted-Root': quote(
+                    broker.root_path),
+                'X-Container-Sysmeta-Sharding': 'True',
+                'X-Backend-Auto-Create': 'True'}
+            # NB: we *used* to send along
+            #    'X-Container-Sysmeta-Shard-Root': broker.root_path
+            # but that isn't safe for container names with nulls or newlines
+            # (or possibly some other characters). We consciously *don't* make
+            # any attempt to set the old meta; during an upgrade, some shards
+            # may think they are in fact roots, but it cleans up well enough
+            # once everyone's upgraded.
             success = self._send_shard_ranges(
                 shard_range.account, shard_range.container,
                 [shard_range], headers=headers)
@@ -1131,7 +1202,7 @@ class ContainerSharder(ContainerReplicator):
             else:
                 self.logger.error(
                     'PUT of new shard container %r failed for %s.',
-                    shard_range, broker.path)
+                    shard_range, quote(broker.path))
                 self._increment_stat('created', 'failure', statsd=True)
                 # break, not continue, because elsewhere it is assumed that
                 # finding and cleaving shard ranges progresses linearly, so we
@@ -1152,18 +1223,22 @@ class ContainerSharder(ContainerReplicator):
 
     def _cleave_shard_range(self, broker, cleaving_context, shard_range):
         self.logger.info("Cleaving '%s' from row %s into %s for %r",
-                         broker.path, cleaving_context.last_cleave_to_row,
-                         shard_range.name, shard_range)
+                         quote(broker.path),
+                         cleaving_context.last_cleave_to_row,
+                         quote(shard_range.name), shard_range)
         self._increment_stat('cleaved', 'attempted')
         start = time.time()
         policy_index = broker.storage_policy_index
         try:
-            shard_part, shard_broker, node_id = self._get_shard_broker(
-                shard_range, broker.root_path, policy_index)
+            shard_part, shard_broker, node_id, put_timestamp = \
+                self._get_shard_broker(shard_range, broker.root_path,
+                                       policy_index)
         except DeviceUnavailable as duex:
             self.logger.warning(str(duex))
             self._increment_stat('cleaved', 'failure', statsd=True)
-            return False
+            return CLEAVE_FAILED
+
+        own_shard_range = broker.get_own_shard_range()
 
         # only cleave from the retiring db - misplaced objects handler will
         # deal with any objects in the fresh db
@@ -1174,13 +1249,34 @@ class ContainerSharder(ContainerReplicator):
         source_db_id = source_broker.get_info()['id']
         source_max_row = source_broker.get_max_row()
         sync_point = shard_broker.get_sync(source_db_id)
-        if sync_point < source_max_row:
+        if sync_point < source_max_row or source_max_row == -1:
             sync_from_row = max(cleaving_context.last_cleave_to_row or -1,
                                 sync_point)
+            objects = None
             for objects, info in self.yield_objects(
                     source_broker, shard_range,
                     since_row=sync_from_row):
                 shard_broker.merge_items(objects)
+            if objects is None:
+                self.logger.info("Cleaving '%s': %r - zero objects found",
+                                 quote(broker.path), shard_range)
+                if shard_broker.get_info()['put_timestamp'] == put_timestamp:
+                    # This was just created; don't need to replicate this
+                    # SR because there was nothing there. So cleanup and
+                    # remove the shard_broker from its hand off location.
+                    self.delete_db(shard_broker)
+                    cleaving_context.range_done(shard_range.upper_str)
+                    if shard_range.upper >= own_shard_range.upper:
+                        # cleaving complete
+                        cleaving_context.cleaving_done = True
+                    cleaving_context.store(broker)
+                    # Because nothing was here we wont count it in the shard
+                    # batch count.
+                    return CLEAVE_EMPTY
+                # Else, it wasn't newly created by us, and
+                # we don't know what's in it or why. Let it get
+                # replicated and counted in the batch count.
+
             # Note: the max row stored as a sync point is sampled *before*
             # objects are yielded to ensure that is less than or equal to
             # the last yielded row. Other sync points are also copied from the
@@ -1193,9 +1289,7 @@ class ContainerSharder(ContainerReplicator):
                 source_broker.get_syncs())
         else:
             self.logger.debug("Cleaving '%s': %r - shard db already in sync",
-                              broker.path, shard_range)
-
-        own_shard_range = broker.get_own_shard_range()
+                              quote(broker.path), shard_range)
 
         replication_quorum = self.existing_shard_replication_quorum
         if shard_range.includes(own_shard_range):
@@ -1204,7 +1298,7 @@ class ContainerSharder(ContainerReplicator):
             # will atomically update its namespace *and* delete the donor.
             # Don't do this when sharding a shard because the donor
             # namespace should not be deleted until all shards are cleaved.
-            if own_shard_range.update_state(ShardRange.SHARDED):
+            if own_shard_range.update_state(ShardRange.SHRUNK):
                 own_shard_range.set_deleted()
                 broker.merge_shard_ranges(own_shard_range)
             shard_broker.merge_shard_ranges(own_shard_range)
@@ -1222,7 +1316,7 @@ class ContainerSharder(ContainerReplicator):
 
         self.logger.info(
             'Replicating new shard container %s for %s',
-            shard_broker.path, shard_broker.get_own_shard_range())
+            quote(shard_broker.path), shard_broker.get_own_shard_range())
 
         success, responses = self._replicate_object(
             shard_part, shard_broker.db_file, node_id)
@@ -1235,34 +1329,32 @@ class ContainerSharder(ContainerReplicator):
             # until each shard range has been successfully cleaved
             self.logger.warning(
                 'Failed to sufficiently replicate cleaved shard %s for %s: '
-                '%s successes, %s required.', shard_range, broker.path,
+                '%s successes, %s required.', shard_range, quote(broker.path),
                 replication_successes, replication_quorum)
             self._increment_stat('cleaved', 'failure', statsd=True)
-            return False
+            return CLEAVE_FAILED
 
         elapsed = round(time.time() - start, 3)
         self._min_stat('cleaved', 'min_time', elapsed)
         self._max_stat('cleaved', 'max_time', elapsed)
         broker.merge_shard_ranges(shard_range)
-        cleaving_context.cursor = shard_range.upper_str
-        cleaving_context.ranges_done += 1
-        cleaving_context.ranges_todo -= 1
+        cleaving_context.range_done(shard_range.upper_str)
         if shard_range.upper >= own_shard_range.upper:
             # cleaving complete
             cleaving_context.cleaving_done = True
         cleaving_context.store(broker)
         self.logger.info(
             'Cleaved %s for shard range %s in %gs.',
-            broker.path, shard_range, elapsed)
+            quote(broker.path), shard_range, elapsed)
         self._increment_stat('cleaved', 'success', statsd=True)
-        return True
+        return CLEAVE_SUCCESS
 
     def _cleave(self, broker):
         # Returns True if misplaced objects have been moved and the entire
         # container namespace has been successfully cleaved, False otherwise
         if broker.is_sharded():
-            self.logger.debug('Passing over already sharded container %s/%s',
-                              broker.account, broker.container)
+            self.logger.debug('Passing over already sharded container %s',
+                              quote(broker.path))
             return True
 
         cleaving_context = CleavingContext.load(broker)
@@ -1272,7 +1364,7 @@ class ContainerSharder(ContainerReplicator):
             # the *retiring* db.
             self.logger.debug(
                 'Moving any misplaced objects from sharding container: %s',
-                broker.path)
+                quote(broker.path))
             bounds = self._make_default_misplaced_object_bounds(broker)
             cleaving_context.misplaced_done = self._move_misplaced_objects(
                 broker, src_broker=broker.get_brokers()[0],
@@ -1281,7 +1373,7 @@ class ContainerSharder(ContainerReplicator):
 
         if cleaving_context.cleaving_done:
             self.logger.debug('Cleaving already complete for container %s',
-                              broker.path)
+                              quote(broker.path))
             return cleaving_context.misplaced_done
 
         ranges_todo = broker.get_shard_ranges(marker=cleaving_context.marker)
@@ -1292,34 +1384,48 @@ class ContainerSharder(ContainerReplicator):
             self.logger.debug('Continuing to cleave (%s done, %s todo): %s',
                               cleaving_context.ranges_done,
                               cleaving_context.ranges_todo,
-                              broker.path)
+                              quote(broker.path))
         else:
             cleaving_context.start()
             cleaving_context.ranges_todo = len(ranges_todo)
             self.logger.debug('Starting to cleave (%s todo): %s',
-                              cleaving_context.ranges_todo, broker.path)
+                              cleaving_context.ranges_todo, quote(broker.path))
 
         ranges_done = []
-        for shard_range in ranges_todo[:self.cleave_batch_size]:
-            if shard_range.state == ShardRange.FOUND:
-                break
+        for shard_range in ranges_todo:
+            if shard_range.state == ShardRange.SHRINKING:
+                # Ignore shrinking shard ranges: we never want to cleave
+                # objects to a shrinking shard. Shrinking shard ranges are to
+                # be expected in a root; shrinking shard ranges (other than own
+                # shard range) are not normally expected in a shard but can
+                # occur if there is an overlapping shard range that has been
+                # discovered from the root.
+                cleaving_context.range_done(None)  # don't move the cursor
+                continue
             elif shard_range.state in (ShardRange.CREATED,
                                        ShardRange.CLEAVED,
                                        ShardRange.ACTIVE):
-                if self._cleave_shard_range(
-                        broker, cleaving_context, shard_range):
+                cleave_result = self._cleave_shard_range(
+                    broker, cleaving_context, shard_range)
+                if cleave_result == CLEAVE_SUCCESS:
                     ranges_done.append(shard_range)
-                else:
+                    if len(ranges_done) == self.cleave_batch_size:
+                        break
+                elif cleave_result == CLEAVE_FAILED:
                     break
+                # else, no errors, but no rows found either. keep going,
+                # and don't count it against our batch size
             else:
-                self.logger.warning('Unexpected shard range state for cleave',
-                                    shard_range.state)
+                self.logger.info('Stopped cleave at unready %s', shard_range)
                 break
 
         if not ranges_done:
+            # _cleave_shard_range always store()s the context on success; make
+            # sure we *also* do that if we hit a failure right off the bat
             cleaving_context.store(broker)
         self.logger.debug(
-            'Cleaved %s shard ranges for %s', len(ranges_done), broker.path)
+            'Cleaved %s shard ranges for %s',
+            len(ranges_done), quote(broker.path))
         return (cleaving_context.misplaced_done and
                 cleaving_context.cleaving_done)
 
@@ -1334,7 +1440,12 @@ class ContainerSharder(ContainerReplicator):
             for sr in modified_shard_ranges:
                 sr.update_state(ShardRange.ACTIVE)
             own_shard_range = broker.get_own_shard_range()
-            own_shard_range.update_state(ShardRange.SHARDED)
+            if own_shard_range.state in (ShardRange.SHRINKING,
+                                         ShardRange.SHRUNK):
+                next_state = ShardRange.SHRUNK
+            else:
+                next_state = ShardRange.SHARDED
+            own_shard_range.update_state(next_state)
             own_shard_range.update_meta(0, 0)
             if (not broker.is_root_container() and not
                     own_shard_range.deleted):
@@ -1348,11 +1459,11 @@ class ContainerSharder(ContainerReplicator):
             else:
                 self.logger.warning(
                     'Failed to remove retiring db file for %s',
-                    broker.path)
+                    quote(broker.path))
         else:
             self.logger.warning(
-                'Repeat cleaving required for %r with context: %s'
-                % (broker.db_files[0], dict(cleaving_context)))
+                'Repeat cleaving required for %r with context: %s',
+                broker.db_files[0], dict(cleaving_context))
             cleaving_context.reset()
             cleaving_context.store(broker)
 
@@ -1362,14 +1473,14 @@ class ContainerSharder(ContainerReplicator):
         candidates = find_sharding_candidates(
             broker, self.shard_container_threshold, shard_ranges)
         if candidates:
-            self.logger.debug('Identified %s sharding candidates'
-                              % len(candidates))
+            self.logger.debug('Identified %s sharding candidates',
+                              len(candidates))
             broker.merge_shard_ranges(candidates)
 
     def _find_and_enable_shrinking_candidates(self, broker):
         if not broker.is_sharded():
             self.logger.warning('Cannot shrink a not yet sharded container %s',
-                                broker.path)
+                                quote(broker.path))
             return
 
         merge_pairs = find_shrinking_candidates(
@@ -1400,7 +1511,7 @@ class ContainerSharder(ContainerReplicator):
 
     def _update_root_container(self, broker):
         own_shard_range = broker.get_own_shard_range(no_default=True)
-        if not own_shard_range:
+        if not own_shard_range or own_shard_range.reported:
             return
 
         # persist the reported shard metadata
@@ -1410,15 +1521,18 @@ class ContainerSharder(ContainerReplicator):
             include_own=True,
             include_deleted=True)
         # send everything
-        self._send_shard_ranges(
-            broker.root_account, broker.root_container,
-            shard_ranges)
+        if self._send_shard_ranges(
+                broker.root_account, broker.root_container, shard_ranges):
+            # on success, mark ourselves as reported so we don't keep
+            # hammering the root
+            own_shard_range.reported = True
+            broker.merge_shard_ranges(own_shard_range)
 
     def _process_broker(self, broker, node, part):
         broker.get_info()  # make sure account/container are populated
         state = broker.get_db_state()
         self.logger.debug('Starting processing %s state %s',
-                          broker.path, state)
+                          quote(broker.path), state)
 
         if not self._audit_container(broker):
             return
@@ -1442,7 +1556,8 @@ class ContainerSharder(ContainerReplicator):
             own_shard_range = broker.get_own_shard_range()
             if own_shard_range.state in (ShardRange.SHARDING,
                                          ShardRange.SHRINKING,
-                                         ShardRange.SHARDED):
+                                         ShardRange.SHARDED,
+                                         ShardRange.SHRUNK):
                 if broker.get_shard_ranges():
                     # container has been given shard ranges rather than
                     # found them e.g. via replication or a shrink event
@@ -1454,8 +1569,8 @@ class ContainerSharder(ContainerReplicator):
                 else:
                     self.logger.debug(
                         'Own shard range in state %r but no shard ranges '
-                        'and not leader; remaining unsharded: %s'
-                        % (own_shard_range.state_text, broker.path))
+                        'and not leader; remaining unsharded: %s',
+                        own_shard_range.state_text, quote(broker.path))
 
         if state == SHARDING:
             if is_leader:
@@ -1474,13 +1589,14 @@ class ContainerSharder(ContainerReplicator):
             cleave_complete = self._cleave(broker)
 
             if cleave_complete:
-                self.logger.info('Completed cleaving of %s', broker.path)
+                self.logger.info('Completed cleaving of %s',
+                                 quote(broker.path))
                 if self._complete_sharding(broker):
                     state = SHARDED
                     self._increment_stat('visited', 'completed', statsd=True)
                 else:
                     self.logger.debug('Remaining in sharding state %s',
-                                      broker.path)
+                                      quote(broker.path))
 
         if state == SHARDED and broker.is_root_container():
             if is_leader:
@@ -1501,9 +1617,8 @@ class ContainerSharder(ContainerReplicator):
             # simultaneously become deleted.
             self._update_root_container(broker)
 
-        self.logger.debug('Finished processing %s/%s state %s',
-                          broker.account, broker.container,
-                          broker.get_db_state())
+        self.logger.debug('Finished processing %s state %s',
+                          quote(broker.path), broker.get_db_state())
 
     def _one_shard_cycle(self, devices_to_shard, partitions_to_shard):
         """
@@ -1517,6 +1632,7 @@ class ContainerSharder(ContainerReplicator):
             - if not a root container, reports shard range stats to the root
               container
         """
+
         self.logger.info('Container sharder cycle starting, auto-sharding %s',
                          self.auto_shard)
         if isinstance(devices_to_shard, (list, tuple)):
@@ -1583,8 +1699,14 @@ class ContainerSharder(ContainerReplicator):
 
         self._report_stats()
 
+    def _set_auto_shard_from_command_line(self, **kwargs):
+        auto_shard = kwargs.get('auto_shard', None)
+        if auto_shard is not None:
+            self.auto_shard = config_true_value(auto_shard)
+
     def run_forever(self, *args, **kwargs):
         """Run the container sharder until stopped."""
+        self._set_auto_shard_from_command_line(**kwargs)
         self.reported = time.time()
         time.sleep(random() * self.interval)
         while True:
@@ -1607,6 +1729,7 @@ class ContainerSharder(ContainerReplicator):
         override_options = parse_override_options(once=True, **kwargs)
         devices_to_shard = override_options.devices or Everything()
         partitions_to_shard = override_options.partitions or Everything()
+        self._set_auto_shard_from_command_line(**kwargs)
         begin = self.reported = time.time()
         self._one_shard_cycle(devices_to_shard=devices_to_shard,
                               partitions_to_shard=partitions_to_shard)
