@@ -15,7 +15,8 @@
 
 from time import time
 from unittest import main, TestCase
-from test.unit import FakeRing, mocked_http_conn, debug_logger
+from test.unit import FakeRing, mocked_http_conn, debug_logger, \
+    make_timestamp_iter
 from tempfile import mkdtemp
 from shutil import rmtree
 from collections import defaultdict
@@ -49,14 +50,17 @@ class FakeInternalClient(object):
         """
         :param aco_dict: A dict of account ,container, object that
             FakeInternalClient can return when each method called. Each account
-            has container name dict, and each container dict has object name
-            list in the container.
+            has container name dict, and each container dict has a list of
+            objects in the container.
             e.g. {'account1': {
-                      'container1: ['obj1', 'obj2', 'obj3'],
+                      'container1: ['obj1', 'obj2', {'name': 'obj3'}],
                       'container2: [],
                       },
                   'account2': {},
                  }
+            N.B. the objects entries should be the container-server JSON style
+            db rows, but this fake will dynamically detect when names are given
+            and wrap them for convenience.
         """
         self.aco_dict = defaultdict(dict)
         self.aco_dict.update(aco_dict)
@@ -69,8 +73,9 @@ class FakeInternalClient(object):
 
     def iter_containers(self, account, prefix=''):
         acc_dict = self.aco_dict[account]
-        return sorted([{'name': six.text_type(container)} for container in
-                       acc_dict if container.startswith(prefix)])
+        return [{'name': six.text_type(container)}
+                for container in sorted(acc_dict)
+                if container.startswith(prefix)]
 
     def delete_container(*a, **kw):
         pass
@@ -78,7 +83,12 @@ class FakeInternalClient(object):
     def iter_objects(self, account, container):
         acc_dict = self.aco_dict[account]
         obj_iter = acc_dict.get(container, [])
-        return [{'name': six.text_type(obj)} for obj in obj_iter]
+        resp = []
+        for obj in obj_iter:
+            if not isinstance(obj, dict):
+                obj = {'name': six.text_type(obj)}
+            resp.append(obj)
+        return resp
 
     def make_request(*a, **kw):
         pass
@@ -101,6 +111,7 @@ class TestObjectExpirer(TestCase):
         self.conf = {'recon_cache_path': self.rcache}
         self.logger = debug_logger('test-expirer')
 
+        self.ts = make_timestamp_iter()
         self.past_time = str(int(time() - 86400))
         self.future_time = str(int(time() + 86400))
         # Dummy task queue for test
@@ -131,9 +142,11 @@ class TestObjectExpirer(TestCase):
 
         # target object paths which should be expirerd now
         self.expired_target_path_list = [
-            'a0/c0/o0', 'a1/c1/o1', 'a2/c2/o2', 'a3/c3/o3', 'a4/c4/o4',
-            'a5/c5/o5', 'a6/c6/o6', 'a7/c7/o7',
-            'a8/c8/o8\xe2\x99\xa1', 'a9/c9/o9\xc3\xb8',
+            swob.wsgi_to_str(tgt) for tgt in (
+                'a0/c0/o0', 'a1/c1/o1', 'a2/c2/o2', 'a3/c3/o3', 'a4/c4/o4',
+                'a5/c5/o5', 'a6/c6/o6', 'a7/c7/o7',
+                'a8/c8/o8\xe2\x99\xa1', 'a9/c9/o9\xc3\xb8',
+            )
         ]
 
     def tearDown(self):
@@ -251,7 +264,8 @@ class TestObjectExpirer(TestCase):
                 self.deleted_objects = {}
 
             def delete_object(self, target_path, delete_timestamp,
-                              task_account, task_container, task_object):
+                              task_account, task_container, task_object,
+                              is_async_delete):
                 if task_container not in self.deleted_objects:
                     self.deleted_objects[task_container] = set()
                 self.deleted_objects[task_container].add(task_object)
@@ -300,9 +314,10 @@ class TestObjectExpirer(TestCase):
             with mock.patch.object(x, 'delete_actual_object',
                                    side_effect=exc) as delete_actual:
                 with mock.patch.object(x, 'pop_queue') as pop_queue:
-                    x.delete_object(actual_obj, ts, account, container, obj)
+                    x.delete_object(actual_obj, ts, account, container, obj,
+                                    False)
 
-            delete_actual.assert_called_once_with(actual_obj, ts)
+            delete_actual.assert_called_once_with(actual_obj, ts, False)
             log_lines = x.logger.get_lines_for_level('error')
             if should_pop:
                 pop_queue.assert_called_once_with(account, container, obj)
@@ -374,13 +389,14 @@ class TestObjectExpirer(TestCase):
         assert_parse_task_obj('1000-a/c/o', 1000, 'a', 'c', 'o')
         assert_parse_task_obj('0000-acc/con/obj', 0, 'acc', 'con', 'obj')
 
-    def make_task(self, delete_at, target):
+    def make_task(self, delete_at, target, is_async_delete=False):
         return {
             'task_account': '.expiring_objects',
             'task_container': delete_at,
             'task_object': delete_at + '-' + target,
             'delete_timestamp': Timestamp(delete_at),
             'target_path': target,
+            'is_async_delete': is_async_delete,
         }
 
     def test_round_robin_order(self):
@@ -588,6 +604,50 @@ class TestObjectExpirer(TestCase):
                 task_account_container_list, my_index, divisor)),
             expected)
 
+        # test some of that async delete
+        async_delete_aco_dict = {
+            '.expiring_objects': {
+                # this task container will be checked
+                self.past_time: [
+                    # tasks ready for execution
+                    {'name': self.past_time + '-a0/c0/o0',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a1/c1/o1',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a2/c2/o2',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a3/c3/o3',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a4/c4/o4',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a5/c5/o5',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a6/c6/o6',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + '-a7/c7/o7',
+                     'content_type': 'application/async-deleted'},
+                    # task objects for unicode test
+                    {'name': self.past_time + u'-a8/c8/o8\u2661',
+                     'content_type': 'application/async-deleted'},
+                    {'name': self.past_time + u'-a9/c9/o9\xf8',
+                     'content_type': 'application/async-deleted'},
+                ]
+            }
+        }
+        async_delete_fake_swift = FakeInternalClient(async_delete_aco_dict)
+        x = expirer.ObjectExpirer(self.conf, logger=self.logger,
+                                  swift=async_delete_fake_swift)
+
+        expected = [
+            self.make_task(self.past_time, target_path,
+                           is_async_delete=True)
+            for target_path in self.expired_target_path_list]
+
+        self.assertEqual(
+            list(x.iter_task_to_expire(
+                task_account_container_list, my_index, divisor)),
+            expected)
+
     def test_run_once_unicode_problem(self):
         requests = []
 
@@ -617,7 +677,7 @@ class TestObjectExpirer(TestCase):
         # executed tasks are with past time
         self.assertEqual(
             mock_method.call_args_list,
-            [mock.call(target_path, self.past_time)
+            [mock.call(target_path, self.past_time, False)
              for target_path in self.expired_target_path_list])
 
     def test_failed_delete_keeps_entry(self):
@@ -635,7 +695,7 @@ class TestObjectExpirer(TestCase):
 
         # all tasks are done
         with mock.patch.object(self.expirer, 'delete_actual_object',
-                               lambda o, t: None), \
+                               lambda o, t, b: None), \
                 mock.patch.object(self.expirer, 'pop_queue') as mock_method:
             self.expirer.run_once()
 
@@ -650,35 +710,36 @@ class TestObjectExpirer(TestCase):
         self.assertEqual(self.expirer.report_objects, 0)
         with mock.patch('swift.obj.expirer.MAX_OBJECTS_TO_CACHE', 0), \
                 mock.patch.object(self.expirer, 'delete_actual_object',
-                                  lambda o, t: None), \
+                                  lambda o, t, b: None), \
                 mock.patch.object(self.expirer, 'pop_queue',
                                   lambda a, c, o: None):
             self.expirer.run_once()
         self.assertEqual(self.expirer.report_objects, 10)
 
-    def test_delete_actual_object_does_not_get_unicode(self):
-        got_unicode = [False]
+    def test_delete_actual_object_gets_native_string(self):
+        got_str = [False]
 
-        def delete_actual_object_test_for_unicode(actual_obj, timestamp):
-            if isinstance(actual_obj, six.text_type):
-                got_unicode[0] = True
+        def delete_actual_object_test_for_string(actual_obj, timestamp,
+                                                 is_async_delete):
+            if isinstance(actual_obj, str):
+                got_str[0] = True
 
         self.assertEqual(self.expirer.report_objects, 0)
 
         with mock.patch.object(self.expirer, 'delete_actual_object',
-                               delete_actual_object_test_for_unicode), \
+                               delete_actual_object_test_for_string), \
                 mock.patch.object(self.expirer, 'pop_queue',
                                   lambda a, c, o: None):
             self.expirer.run_once()
 
         self.assertEqual(self.expirer.report_objects, 10)
-        self.assertFalse(got_unicode[0])
+        self.assertTrue(got_str[0])
 
     def test_failed_delete_continues_on(self):
         def fail_delete_container(*a, **kw):
             raise Exception('failed to delete container')
 
-        def fail_delete_actual_object(actual_obj, timestamp):
+        def fail_delete_actual_object(actual_obj, timestamp, is_async_delete):
             raise Exception('failed to delete actual object')
 
         with mock.patch.object(self.fake_swift, 'delete_container',
@@ -713,19 +774,12 @@ class TestObjectExpirer(TestCase):
         interval = 1234
         x = expirer.ObjectExpirer({'__file__': 'unit_test',
                                    'interval': interval})
-        orig_random = expirer.random
-        orig_sleep = expirer.sleep
-        try:
-            expirer.random = not_random
-            expirer.sleep = not_sleep
+        with mock.patch.object(expirer, 'random', not_random), \
+                mock.patch.object(expirer, 'sleep', not_sleep), \
+                self.assertRaises(SystemExit) as caught:
             x.run_once = raise_system_exit
             x.run_forever()
-        except SystemExit as err:
-            pass
-        finally:
-            expirer.random = orig_random
-            expirer.sleep = orig_sleep
-        self.assertEqual(str(err), 'test_run_forever')
+        self.assertEqual(str(caught.exception), 'test_run_forever')
         self.assertEqual(last_not_sleep, 0.5 * interval)
 
     def test_run_forever_catches_usual_exceptions(self):
@@ -765,10 +819,30 @@ class TestObjectExpirer(TestCase):
 
         x = expirer.ObjectExpirer({})
         ts = Timestamp('1234')
-        x.delete_actual_object('/path/to/object', ts)
+        x.delete_actual_object('/path/to/object', ts, False)
         self.assertEqual(got_env[0]['HTTP_X_IF_DELETE_AT'], ts)
         self.assertEqual(got_env[0]['HTTP_X_TIMESTAMP'],
                          got_env[0]['HTTP_X_IF_DELETE_AT'])
+        self.assertEqual(
+            got_env[0]['HTTP_X_BACKEND_CLEAN_EXPIRING_OBJECT_QUEUE'], 'no')
+
+    def test_delete_actual_object_bulk(self):
+        got_env = [None]
+
+        def fake_app(env, start_response):
+            got_env[0] = env
+            start_response('204 No Content', [('Content-Length', '0')])
+            return []
+
+        internal_client.loadapp = lambda *a, **kw: fake_app
+
+        x = expirer.ObjectExpirer({})
+        ts = Timestamp('1234')
+        x.delete_actual_object('/path/to/object', ts, True)
+        self.assertNotIn('HTTP_X_IF_DELETE_AT', got_env[0])
+        self.assertNotIn('HTTP_X_BACKEND_CLEAN_EXPIRING_OBJECT_QUEUE',
+                         got_env[0])
+        self.assertEqual(got_env[0]['HTTP_X_TIMESTAMP'], ts.internal)
 
     def test_delete_actual_object_nourlquoting(self):
         # delete_actual_object should not do its own url quoting because
@@ -784,12 +858,41 @@ class TestObjectExpirer(TestCase):
 
         x = expirer.ObjectExpirer({})
         ts = Timestamp('1234')
-        x.delete_actual_object('/path/to/object name', ts)
+        x.delete_actual_object('/path/to/object name', ts, False)
         self.assertEqual(got_env[0]['HTTP_X_IF_DELETE_AT'], ts)
         self.assertEqual(got_env[0]['HTTP_X_TIMESTAMP'],
                          got_env[0]['HTTP_X_IF_DELETE_AT'])
         self.assertEqual(got_env[0]['PATH_INFO'], '/v1/path/to/object name')
 
+    def test_delete_actual_object_async_returns_expected_error(self):
+        def do_test(test_status, should_raise):
+            calls = [0]
+
+            def fake_app(env, start_response):
+                calls[0] += 1
+                calls.append(env['PATH_INFO'])
+                start_response(test_status, [('Content-Length', '0')])
+                return []
+
+            internal_client.loadapp = lambda *a, **kw: fake_app
+
+            x = expirer.ObjectExpirer({})
+            ts = Timestamp('1234')
+            if should_raise:
+                with self.assertRaises(internal_client.UnexpectedResponse):
+                    x.delete_actual_object('/path/to/object', ts, True)
+            else:
+                x.delete_actual_object('/path/to/object', ts, True)
+            self.assertEqual(calls[0], 1, calls)
+
+        # object was deleted and tombstone reaped
+        do_test('404 Not Found', False)
+        # object was overwritten *after* the original delete, or
+        # object was deleted but tombstone still exists, or ...
+        do_test('409 Conflict', False)
+        # Anything else, raise
+        do_test('400 Bad Request', True)
+
     def test_delete_actual_object_returns_expected_error(self):
         def do_test(test_status, should_raise):
             calls = [0]
@@ -805,9 +908,9 @@ class TestObjectExpirer(TestCase):
             ts = Timestamp('1234')
             if should_raise:
                 with self.assertRaises(internal_client.UnexpectedResponse):
-                    x.delete_actual_object('/path/to/object', ts)
+                    x.delete_actual_object('/path/to/object', ts, False)
             else:
-                x.delete_actual_object('/path/to/object', ts)
+                x.delete_actual_object('/path/to/object', ts, False)
             self.assertEqual(calls[0], 1)
 
         # object was deleted and tombstone reaped
@@ -832,7 +935,7 @@ class TestObjectExpirer(TestCase):
         x = expirer.ObjectExpirer({})
         exc = None
         try:
-            x.delete_actual_object('/path/to/object', Timestamp('1234'))
+            x.delete_actual_object('/path/to/object', Timestamp('1234'), False)
         except Exception as err:
             exc = err
         finally:
@@ -845,7 +948,7 @@ class TestObjectExpirer(TestCase):
         x = expirer.ObjectExpirer({})
         x.swift.make_request = mock.Mock()
         x.swift.make_request.return_value.status_int = 204
-        x.delete_actual_object(name, timestamp)
+        x.delete_actual_object(name, timestamp, False)
         self.assertEqual(x.swift.make_request.call_count, 1)
         self.assertEqual(x.swift.make_request.call_args[0][1],
                          '/v1/' + urllib.parse.quote(name))
@@ -855,7 +958,7 @@ class TestObjectExpirer(TestCase):
         timestamp = Timestamp('1515544858.80602')
         x = expirer.ObjectExpirer({})
         x.swift.make_request = mock.MagicMock()
-        x.delete_actual_object(name, timestamp)
+        x.delete_actual_object(name, timestamp, False)
         self.assertEqual(x.swift.make_request.call_count, 1)
         header = 'X-Backend-Clean-Expiring-Object-Queue'
         self.assertEqual(
@@ -872,7 +975,8 @@ class TestObjectExpirer(TestCase):
         with mocked_http_conn(
                 200, 200, 200, give_connect=capture_requests) as fake_conn:
             x.pop_queue('a', 'c', 'o')
-            self.assertRaises(StopIteration, fake_conn.code_iter.next)
+            with self.assertRaises(StopIteration):
+                next(fake_conn.code_iter)
         for method, path in requests:
             self.assertEqual(method, 'DELETE')
             device, part, account, container, obj = utils.split_path(
@@ -881,6 +985,15 @@ class TestObjectExpirer(TestCase):
             self.assertEqual(container, 'c')
             self.assertEqual(obj, 'o')
 
+    def test_build_task_obj_round_trip(self):
+        ts = next(self.ts)
+        a = 'a1'
+        c = 'c2'
+        o = 'obj1'
+        args = (ts, a, c, o)
+        self.assertEqual(args, expirer.parse_task_obj(
+            expirer.build_task_obj(ts, a, c, o)))
+
 
 if __name__ == '__main__':
     main()
